package com.V10lator.SignClock;

import java.util.Collection;
import java.util.HashMap;
import java.util.logging.Logger;
import org.bukkit.block.Block;
import org.bukkit.entity.Player;
import org.bukkit.event.Event;
import org.bukkit.plugin.PluginDescriptionFile;
import org.bukkit.plugin.PluginManager;
import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.Server;

import com.V10lator.SignClock.themeLib;
import com.V10lator.SignClock.triggerRAM;
import com.V10lator.lib24time.l24t;

public class SignClock extends JavaPlugin
{
  private Logger log = Logger.getLogger("Minecraft");
  public String name;
  private final SignClockBlockListener blockListener = new SignClockBlockListener(this);
  private final SignClockPlayerListener playerListener = new SignClockPlayerListener(this);
  private final HashMap<Block, SClock> signs = new HashMap<Block, SClock>();
  public final themeLib themeLib = new themeLib(this);
  public final triggerRAM triggerRAM = new triggerRAM();
  public final l24t l24t = new l24t();
  private final MagicSign magicSign = new MagicSign();
  public Init Init;

  public void onEnable()
  {
	int saveIntervall = 15;
	Server server = getServer();
    PluginManager pm = server.getPluginManager();
    
    int major = l24t.getMajor();
	int minor = l24t.getMinor();
	if(minor < 4)
	{
	  info2log("ERROR: lib24time >= 0.4 needed (" + major + "." + minor + " found)");
	  pm.disablePlugin(this);
	  return;
	}
    
    pm.registerEvent(Event.Type.SIGN_CHANGE, blockListener, Event.Priority.Normal, this);
    pm.registerEvent(Event.Type.REDSTONE_CHANGE, blockListener, Event.Priority.Normal, this);
    pm.registerEvent(Event.Type.BLOCK_BREAK, blockListener, Event.Priority.Normal, this);
    pm.registerEvent(Event.Type.PLAYER_INTERACT, playerListener, Event.Priority.Normal, this);
    
    PluginDescriptionFile pdfFile = this.getDescription();
    this.name = pdfFile.getName();

    Init = new Init(this);
    
    if(this.Init.initialize((short)0) < 0)
    {
      pm.disablePlugin(this);
      return;
    }
    
    if(!themeLib.initialize())
    {
      pm.disablePlugin(this);
      return;
    }
    
    this.getServer().getScheduler().scheduleSyncRepeatingTask(this, new frameTick(this), 1, 16);
    this.getServer().getScheduler().scheduleSyncRepeatingTask(this, new triggerTicker(this), 1, 4);
    
    int init = this.Init.initialize((short)1);
    if(init == -1)
    {
      pm.disablePlugin(this);
      return;
    }
    else if(init == -2)
    {
      pm.disablePlugin(this);
      pm.enablePlugin(this);
      return;
    }
    else
      this.info2log(init + " SignClocks resetted");
    
    getServer().getScheduler().scheduleSyncRepeatingTask(this, new SaveThread(this), 60 * 30 * 10, 60 * saveIntervall * 10);
    this.info2log("v" + pdfFile.getVersion() + " enabled");
  }
  
  public void onDisable()
  {
	this.getServer().getScheduler().cancelTasks(this);
	SaveThread st = new SaveThread(this);
	st.run();
    this.info2log("disabled");
  }

  public void info2log(String text)
  {
    log.info("[" + this.name + "] " + text + ".");
  }
  
  public void addSign(Block block, SClock sign)
  {
    if(sign.isTriggered())
      triggerRAM.addSign(block.getWorld().getName(), block.getX(), block.getY(), block.getZ());

    //add Sign to hashmap
    signs.put(block, sign);
  }
	    
  public boolean isSignClock(Block block)
  {
    return signs.containsKey(block);
  }
	    
  public void removeSign(Block block)
  {
	triggerRAM.removeSign(block.getWorld().getName(), block.getX(), block.getY(), block.getZ());
    signs.remove(block);
  }
	    
  public SClock getSignClock(Block block)
  {
    if(!isSignClock(block))
      return null;
	return signs.get(block);
  }
  
  public boolean hasPermission(Player player)
  {
	if(!player.isOnline())
	  return false;
	if(!this.magicSign.isActive())
	  return true;
	if(!player.isOp())
	  return false;
	return true;
  }
  
  public boolean isMagicSign(Block block)
  {
	return this.magicSign.isActive(block);
  }
  public boolean isMagicActive()
  {
	return this.magicSign.isActive();
  }
  public void setMagicSign(Block block)
  {
	this.magicSign.setActive(block);
  }
  public void removeMagicSign(Block block)
  {
	this.magicSign.setInactive(block);
  }
  public void setMagic(boolean magic)
  {
	this.magicSign.setMagic(magic);
  }
  public Block getMS()
  {
    return this.magicSign.getMS();
  }
  
  public Collection<SClock> getValues()
  {
	return this.signs.values();
  }
}