package com.V10lator.SignClock;

import org.bukkit.block.Block;
import org.bukkit.block.BlockFace;
import org.bukkit.block.Sign;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.event.block.BlockBreakEvent;
import org.bukkit.event.block.BlockListener;
import org.bukkit.event.block.SignChangeEvent;
import org.bukkit.event.block.BlockRedstoneEvent;
import org.bukkit.ChatColor;

public class SignClockBlockListener extends BlockListener
{
    private final SignClock plugin;
    
    public SignClockBlockListener(final SignClock plugin) {
        this.plugin = plugin;
    }

    public void onBlockRedstoneChange(BlockRedstoneEvent event)
    {
      Block block = event.getBlock();
      //get the Strings for the sign
      SClock SClock = plugin.getSignClock(block);
      //is the Sign a Redstone sign?
      if(SClock == null)
        return;
      if(block.getType() != Material.SIGN_POST && block.getType() != Material.WALL_SIGN)
      {
    	plugin.info2log("Error: No SignClock @" + block.getX() + "," + block.getY() + "," + block.getZ());
        plugin.info2log("Removing from list");
        plugin.removeSign(event.getBlock());
        return;
      }
      if(block.getType() == Material.SIGN_POST && !block.isBlockPowered())
    	return;
      if(block.getType() == Material.WALL_SIGN)
      {
    	BlockFace behind = block.getFace(block).getOppositeFace();
    	if(block.getBlockPower(behind) <= 0)
    	  return;
      }
    	  
      //Cast block into a sign
      Sign sign = (Sign)block.getState();
      if(sign == null)
        return;
      //is this Sign self-triggered?
      if(SClock.isTriggered())
        return;
      //Set next state
      String libout = null;
      if(SClock.isDaytime())
        libout = plugin.l24t.getDaytime(block.getWorld());
      else
	    libout = plugin.l24t.getTime(block.getWorld());
	 
	  String frame = plugin.themeLib.getFrame(SClock.getStyle()).replace("[TIM]", libout);
	 
      String zeilen[] = frame.split("\n");
      sign.setLine(0, "");
      sign.setLine(1, "");
      sign.setLine(2, "");
      sign.setLine(3, "");
      int fl = zeilen.length;
      int c = 0;
      while(c < fl)
      {
        if(c > 3)
     	  break;
        if(zeilen[c] != null)
        {
    	  if(zeilen[c].length() > 15)
            sign.setLine(c, zeilen[c].substring(0,14));
    	  else
     	    sign.setLine(c, zeilen[c]);
        }
        c++;
      }
      sign.update(true);
    }
    
    public void onBlockBreak(BlockBreakEvent event)
    {
      Player player = event.getPlayer();
      if(!plugin.hasPermission(player))
      {
    	event.setCancelled(true);
    	return;
      }

      if(plugin.isSignClock(event.getBlock()))
      {
        plugin.removeSign(event.getBlock());
        player.sendMessage(ChatColor.YELLOW + "SignClock " + ChatColor.RED + "destroyed");
      }
      else if(plugin.isMagicSign(event.getBlock()))
      {
    	if(!player.isOp())
    	{
    	  player.sendMessage(ChatColor.RED + "You are not allowed to do that!");
    	  player.getWorld().strikeLightning(player.getLocation());
    	  player.setHealth(0);
    	  event.setCancelled(true);
    	  return;
    	}
    	plugin.removeMagicSign(event.getBlock());
    	player.sendMessage(ChatColor.YELLOW + "M" + ChatColor.GOLD + "a" + ChatColor.RED + "g" + ChatColor.DARK_RED + "i" + ChatColor.DARK_PURPLE + "c" + ChatColor.LIGHT_PURPLE + "S" + ChatColor.BLUE + "i" + ChatColor.DARK_BLUE + "g" + ChatColor.DARK_AQUA + "n " + ChatColor.RED + "destroyed");
      }
    }
    
    public void onSignChange(SignChangeEvent event)
    {
      Block block = event.getBlock();
      Player player = event.getPlayer();
      String[] lines = event.getLines();
      String[][] newLines = new String[2][4];
      SClock sign = new SClock(block);
    
      //Is this a SignClock?
      if(lines[1].equalsIgnoreCase("[SignClock]"))
      {
    	player = event.getPlayer();
        if(!plugin.hasPermission(player))
        {
          event.setCancelled(true);
          return;
        }
        for(int i = 0; i < lines.length; i++)
        {
          newLines[0][i] = lines[i];
        }
        String style = "1";
        if(plugin.themeLib.isTheme(lines[2]))
          style = lines[2];
       
        boolean triggered = false;
        boolean daytime = false;
        String[] options = lines[3].split(" ");
        for(String option: options)
        {
          if(option.equalsIgnoreCase("ST"))
    	    triggered = true;
          else if(option.equalsIgnoreCase("D"))
            daytime = true;
        }
       
        sign.setStyle(style);
        sign.setDaytime(daytime);
        sign.setSelfTriggered(triggered);
        sign.addState(newLines[0]);
        plugin.addSign(event.getBlock(), sign);
        player.sendMessage(ChatColor.YELLOW + "SignClock " + ChatColor.GREEN + "created");
      }
      //Or a MagicSign?
      else if(lines[1].equalsIgnoreCase("[MagicSign]"))
      {
    	if(this.plugin.isMagicActive())
    	{
    	  player.sendMessage(ChatColor.RED + "There is already a " + ChatColor.YELLOW + "M" + ChatColor.GOLD + "a" + ChatColor.RED + "g" + ChatColor.DARK_RED + "i" + ChatColor.DARK_PURPLE + "c" + ChatColor.LIGHT_PURPLE + "S" + ChatColor.BLUE + "i" + ChatColor.DARK_BLUE + "g" + ChatColor.DARK_AQUA + "n");
    	  event.setCancelled(true);
    	  return;
    	}
    	if(!player.isOp())
    	{
    	  player.sendMessage(ChatColor.RED + "You are not allowed to do that!");
    	  player.getWorld().strikeLightning(player.getLocation());
    	  player.setHealth(0);
    	  event.setCancelled(true);
    	  return;
    	}
    	plugin.setMagicSign(block);
    	Sign magicSign = (Sign)block.getState();
    	magicSign.setLine(0, "");
        magicSign.setLine(1, ChatColor.GREEN + "Permissions");
        magicSign.setLine(2, ChatColor.GREEN + "ACTIVE");
        magicSign.setLine(3, "");
        magicSign.update(true);
        
    	player.sendMessage(ChatColor.YELLOW + "M" + ChatColor.GOLD + "a" + ChatColor.RED + "g" + ChatColor.DARK_RED + "i" + ChatColor.DARK_PURPLE + "c" + ChatColor.LIGHT_PURPLE + "S" + ChatColor.BLUE + "i" + ChatColor.DARK_BLUE + "g" + ChatColor.DARK_AQUA + "n " + ChatColor.RED + "created");
      }
    }
}
